program boson

! *******************************************************************
! *                                                                 *
! * perform the quantum-mechanical complement to the classical step *
! * of the dielectric theory of eels in specular geometry using a   *
! * suitable thermodynamical average of the quantized surface       *
! * harmonic oscillators                                            *
! *                                                                 *
! *******************************************************************

  implicit none

  integer, parameter :: mmax=14, nmax=2**mmax
  double precision :: asym, emax, emin, gauss, t, width, wmin, wmax, y
  double precision :: p2(nmax)
  integer :: i, np, ioStatus
  character (len = 72) :: comment(2)
  double precision, allocatable :: xout(:), yout(:)
  integer :: nout

! *** read input parameters

1   call change_working_dir()
1   open(unit = 13, file = 'bosin')
! target temperature (Kelvin)
1   read(13, *) t
! width of the instrumental response (cm**-1)
1   read(13, *) width
! fraction of gaussian for the instrumental response
1   read(13, *) gauss
! asymmetry of the instrumental response
1   read(13, *) asym
! lower and upper energy losses for this computation (cm**-1)
1   read(13, *) emin
1   read(13, *) emax
1   close(unit=13)

1   write(*,*) 'program boson (September 2020)'
1   write(*,'(a, f6.1, a, f7.2, a)') ' t =', t, ' K, width =', width, ' cm**-1'
1   write(*,'(a, f5.2, a, f5.2)') ' gauss =', gauss, ', asym =', asym
1   write(*,'(a, g11.4, a, g11.4, a)') ' energy losses from', emin, ' to', emax, ' cm**-1'

! *** read the table of values of the classical loss spectrum

1   open(unit = 12, file = 'eelsou')
1   read(12, '(a48)') comment(1)
1   read(12, '(a72)') comment(2)
1   np = 0
1   do
327     read(12, *, IOSTAT = ioStatus) wmax, y
327     if (ioStatus /= 0) exit
326     if (wmax < 0.0d0) cycle
326     np = np + 1
326     p2(np) = y
326     if (np == 1) wmin = wmax
326     if (np < nmax) cycle
*>   enddo
1   close(unit = 12)

1   if (np <= 0) stop '*** no input values for pcl ***'

1   write(*,*) comment(2)
1   write(*,'(a, i6, a, g15.7, a, g15.7)') ' read', np, ' values of pcl from', wmin, ' to', wmax

! length calculation for xout, yout.
!
1   nout = 2**14
1   allocate (xout(nout))
1   allocate (yout(nout))

1   call doboson(t, width, gauss, asym, emin, emax, wmin, wmax, np, p2, xout, yout, nout)

1   open(unit = 14, file = 'bosou')
1   write(14, '(a, a, f6.1, a, f5.2)') comment(1), 'T =', t, ' GAUSS =', gauss
1   write(14, *) comment(2)
1   do i = 1, nout
851     write(14, '(2e15.7)') xout(i), yout(i)
851   end do
1   close(unit = 14)
1   write(*,*) nout, ' values written on disk'

1   deallocate (xout, yout)
1   stop
end program boson

subroutine change_working_dir()

! This routine gets the first argument of the commandline and takes it
! as the path to change the working directory
! used intrinsic routines:
! iarg returns the number of commandline arguments without the program cname.
! chdir changes the directory and returns 0 on success.
! trim removes blanks from strings.

  character (len = 256) :: argument
  integer :: status

1   if (iargc() == 1) then
*>     call getarg(1, argument)
*>     status = chdir(trim(argument))
*>     if (status /= 0) then
*>       write (*,*) '*** change directory failed ***'
*>       write (*,*) 'directory tried: ', trim(argument)
*>       write (*,*) 'error code (see: man chdir): ', status
*>       write (*,*) 'continuing in the start directory'
*>     end if
*>   end if

1   return
end subroutine change_working_dir

double precision function respon(w, width)

!*******************************************************************
!                                                                  *
! instrumental response of the spectrometer for the frequency w    *
! width is the full width at half maximum of the response function *
!                                                                  *
!*******************************************************************

  implicit none
  double precision, intent(in) :: w, width

  double precision :: u

*>   u = 1.25d0 * w / width
*>   if (u <= -1.0d0 .or. u >= 2.0d0) then
*>      respon = 0.0d0
*>   elseif (u > 1.0d0) then
*>      respon = 4.0d0 + 2 * dsqrt((u - 1.0d0)**3) - 3 * u
*>   elseif (u > 0.0d0) then
*>      respon = 2.0d0 - 4 * dsqrt(u**3) + 3 * u
*>   else
*>      respon = 2 * dsqrt((u + 1.0d0)**3)
*>   endif
*>   return
end function respon

subroutine sicot(f, m, h, x0)

! *******************************************************************
! *                                                                 *
! * integral transform g(y, x0) of a real function f(x) defined by  *
! *                                                                 *
! * g(x0, y) = integral of f(x)/tanh(x/x0)*(1-cos(x*y)) dx          *
! *                                                                 *
! * f(x) is tabulated on the mesh of points xj = (j-1/2)*h ,        *
! * j = 1, 2, ..., n ,  with n = 2**m                               *
! * g(x0, y) is computed on the mesh of points yk = k*2pi/(n*h) ,   *
! * k = 0, 1, 2, ..., n-1                                           *
! *                                                                 *
! * input ...                                                       *
! * f(j) contains the value of f(xj) , j = 1, 2, ..., n             *
! * m is such that n = 2**iabs(m)                                   *
! * h is the step size (must be positive)                           *
! *                                                                 *
! * output ...                                                      *
! * f(k+1) contains g(x0, yk), k = 0, 1, ..., n-1                   *
! *                                                                 *
! * computing remarks ...                                           *
! * f is a real array with dimension n or more                      *
! * it is supposed that f(x) is zero for x > n*h                    *
! * external reference : sintr (sine transform of a real function). *
! *                                                                 *
! *******************************************************************

  implicit none

  double precision, intent(in out) :: f(*)
  integer, intent(in) :: m
  double precision, intent(in) :: h
  double precision, intent(in) :: x0

  double precision :: a, b, c, d, e, s, sa
  integer :: i, j, msign, n

  logical :: debug

  interface
    subroutine sintr(f, msign, h)
      double precision, intent(in out) :: f(*)
      integer, intent(in) :: msign
      double precision, intent(in) :: h
    end subroutine sintr
  end interface  

1   debug = .false.
1   if (m == 0) then
*>     f(1) = 0.0d0
*>     return
*>   endif

1   if (h <= 0.0d0) then
*>     if (debug) write(*,*) ' *** incorrect step size in <sicot>, h =', h, ' ***'
*>     stop
*>   endif

1   if (x0 < 0.0d0) then
*>     if (debug) write(*,*) ' *** incorrect input in <sicot>, x0 =', x0, ' ***'
*>     stop
*>   endif

1   n = 2**iabs(m)

! *** evaluate the integral from xj to infinity of f(x)/tanh(x/x0) dx
! and store the result in f(j)

1   c = 0.0d0
1   if (x0 > h / 16) then
1     c = dexp(-h / x0)
1   endif
1   s = 1.0d0 - c
1   e = c**2
1   a = 0.25d0 * h
1   b = 0.25d0 * x0
1   do i = 1, n
4096     if (i < n) then
4095       f(i) = f(i) + f(i+1)
4095     endif
4096     d = a
4096     if (s /= 1.0d0) then
3902       sa = s
3902       c = c * e
3902       s = 1.0d0 - c
3902       d = d + b * dlog(s / sa)
3902     endif
4096     f(i) = f(i) * d
4096   enddo

1   j = n
1   do i = 2, n
4095     j = j - 1
4095     f(j) = f(j) + f(j + 1)
4095   enddo
! alternative, but not yet tested:
!  do j = n - 1, 1, -1
!    f(j) = f(j) + f(j + 1)
!  enddo
1   msign = -iabs(m)
1   call sintr(f, msign, h)
1   return
end subroutine sicot

subroutine sintr(f, msign, h)

! *******************************************************************
! *                                                                 *
! * integral sine transform of a real function f(x)                 *
! *                                                                 *
! * g(y) = integral from zero to infinity of f(x)*sin(x*y) dx       *
! * if msign >= 0 or                                                *
! * g(y) = y * integral from zero to infinity of f(x)*sin(x*y) dx   *
! * if msign < 0                                                    *
! *                                                                 *
! * f(x) is tabulated on the mesh of points xj = (j-1/2)*h ,        *
! * j = 1, 2, ..., n ,  with n = 2**iabs(msig)                      *
! * g(y) is computed on the mesh of points yk = k*2pi/(n*h) ,       *
! * k = 0, 1, 2, ..., n-1                                           *
! *                                                                 *
! * input ...                                                       *
! * f(j) contains the value of f(xj) , j = 1, 2, ..., n             *
! * msign is such that n = 2**iabs(msign)                           *
! * h is the step size (must be positive)                           *
! *                                                                 *
! * output ...                                                      *
! * f(k+1) contains g(yk), k = 0, 1, ..., n-1                       *
! *                                                                 *
! * computing remarks ...                                           *
! * f is a real array with dimension n or more                      *
! * it is supposed that f(x) is zero for x > n*h                    *
! *                                                                 *
! *******************************************************************

  implicit none

  double precision, intent(in out) :: f(*)
  integer, intent(in) :: msign
  double precision, intent(in) :: h

  double precision :: a, c, ca, d, e, fnp1, s, sa, ti, ti1, ti2, tr, tr1, tr2
  double precision :: ui, ur, wi, wr
  integer :: i, i2, ip2, j, j2, k, l, le, le1, m2, n, n2, nm1, nv2

  logical :: debug

2   debug = .false.
2   if (h <= 0.0d0) then
*>     if (debug) write(*,*) ' *** incorrect step size in <sintr>, h: ', h, ' ***'
*>     stop
*>   endif

2   if (msign == 0) then
*>     f(1) = 0.0d0
*>     return
*>   endif

! *** for j and k = 0, 1, 2 ... n/2-1, compute
! s1 = sum ( f(2*j+1)*cos(4*pi*k*j/n) - f(2*j+2)*sin(4*pi*k*j/n) )
! s2 = sum ( f(2*j+1)*sin(4*pi*k*j/n) + f(2*j+2)*cos(4*pi*k*j/n) )
! store s1 in f(2*k+1) and s2 in f(2*k+2)
! (adapted from cfft routine, cern library member d704)

2   m2 = iabs(msign) - 1
2   n = 2**m2
2   if (n /= 1) then
2     nv2 = n / 2
2     nm1 = n - 1
2     j = 1
2     do i = 1, nm1
4094       if (i < j) then
1984         i2 = i + i
1984         j2 = j + j
1984         ti = f(j2)
1984         f(j2) = f(i2)
1984         f(i2) = ti
1984         i2 = i2 - 1
1984         j2 = j2 - 1
1984         tr = f(j2)
1984         f(j2) = f(i2)
1984         f(i2) = tr
1984       endif
4094       k = nv2
4094       do while (j > k)
4072         j = j - k
4072         k = k / 2
4072       enddo
4094       j = j + k
4094     enddo
2     do i = 1, n, 2
2048       i2 = i + i
2048       ti = f(i2 + 2)
2048       f(i2 + 2) = f(i2) - ti
2048       f(i2) = f(i2) + ti
2048       i2 = i2 - 1
2048       tr = f(i2 + 2)
2048       f(i2 + 2) = f(i2) - tr
2048       f(i2) = f(i2) + tr
2048     enddo
2     if (m2 /= 1) then
2       c = 0.0d0
2       s = 1.0d0
2       le = 2
2       do l = 2, m2
20         wr = c
20         wi = s
20         ur = wr
20         ui = wi
20         c = dsqrt(c * 0.5d0 + 0.5d0 )
20         s = s / (c + c)
20         le1 = le
20         le = le1 + le1
20         do i = 1, n, le
2046           i2 = i + i
2046           ip2 = i2 + le
2046           ti = f(ip2)
2046           f(ip2) = f(i2) - ti
2046           f(i2) = f(i2) + ti
2046           i2 = i2 - 1
2046           ip2 = ip2 - 1
2046           tr = f(ip2)
2046           f(ip2) = f(i2) - tr
2046           f(i2) = f(i2) + tr
2046         enddo
20         do j = 2, le1
4072           do i = j, n, le
18434             i2 = i + i
18434             ip2 = i2 + le
18434             tr = f(ip2 - 1) * ur - f(ip2) * ui
18434             ti = f(ip2) * ur + f(ip2 - 1) * ui
18434             f(ip2) = f(i2) - ti
18434             f(i2) = f(i2) + ti
18434             i2 = i2 - 1
18434             ip2 = ip2 - 1
18434             f(ip2) = f(i2) - tr
18434             f(i2) = f(i2) + tr
18434           enddo
4072           tr = ur * wr - ui * wi
4072           ui = ui * wr + ur * wi
4072           ur = tr
4072         enddo
20       enddo
2     endif
2   endif

! *** for j and k = 0, 1 ... n-1, transform the array f so obtained into
! sum f(j+1)*cos(2*pi*k*j/n) and sum f(j+1)*sin(2*pi*k*j/n)
! and multiply the results by (1 - cos(2*pi*k/n)) and sin(2*pi*k/n)

2   fnp1 = 4 * (f(1) - f(2))
2   a = 3.141592653589793238d0 / n
2   n2 = n + 1
2   n = 2 * n
2   if (n2 /= 2) then
2     c = 1.0d0
2     s = 0.0d0
2     ca = dcos(a)
2     sa = dsin(a)
2     k = n + 1
2     do j = 3, n2, 2
2048       k = k - 2
2048       d = c
2048       c = d * ca - s * sa
2048       s = d * sa + s * ca
2048       tr1 = f(j) + f(k)
2048       ti1 = f(j + 1) - f(k + 1)
2048       d = f(j) - f(k)
2048       e = f(j + 1) + f(k + 1)
2048       tr2 = d * s + e * c
2048       ti2 = e * s - d * c
2048       f(j) = (1.0d0 - c) * (tr1 + tr2)
2048       f(j + 1) = s * (ti1 + ti2)
2048       f(k) = (1.0d0 + c) * (tr1 - tr2)
2048       f(k + 1) = s * (ti2 - ti1)
2048     enddo
2     n2 = n2 - 1
2     do j = 2, n2
4094       j2 = j + j
4094       f(j) = f(j2 - 1) + f(j2)
4094     enddo
2     do j = 2, n2
4094       f(n + 2 - j) = f(j)
4094     enddo
2     n2 = n2 + 1
2   endif
2   f(n2) = fnp1
2   if (msign >= 0) then

! *** normalization

1     c = 0.0d0
1     a = (a + a) / h
1     do j = 2, n
4095       c = c + a
4095       f(j) = f(j) / c
4095     enddo
1   endif
2   f(1) = 0.0d0
2   return
end subroutine sintr

subroutine rcffi(ar, ai, msign, h)

!   *******************************************************************
!   *                                                                 *
!   * using a radix-two fast-fourier-transform technique, rcffi       *
!   * computes the fourier integral transform of a real function      *
!   * f(x) or the inverse fourier transform of a complex function     *
!   * such that g(-y) = conj(g(y)).                                   *
!   * (adapted from cfft routine, cern library member d704)           *
!   *                                                                 *
!   * g(y) = integral f(x) * cexp(-i * x * y) dx    (msign < 0)       *
!   *                                                                 *
!   * f(x) = 1 / (2 * pi) integral g(y) * cexp(+i * y * x) dy  (msign > 0)    *
!   *                                                                 *
!   * f(x) is tabulated on the meshes of points defined by            *
!   * xj = j * hx and -xj = -j * hx,  j = 0, +1, ..., n-2, n-1        *
!   * ar(j+1) contains f(+xj) and ai(j+1) contains f(-xj)             *
!   * (input when msign < 0 , output when msign >0)                   *
!   * ar(1) may be different from ai(1)                               *
!   *                                                                 *
!   * g(y) is tabulated on the mesh of points defined by              *
!   * yk = k * hy  , k = 0, 1, ..., n - 2, n - 1                      *
!   * ar(j+1) contains real(g(yk)) and ai(j+1) contains aimag(g(yk))  *
!   * (output when msign > 0 , input when msign < 0)                  *
!   * remark :  g(-yk) = conj(g(+yk))                                 *
!   *                                                                 *
!   * n = 2**iabs(msign)  (msign is an input)                         *
!   *                                                                 *
!   * the step sizes satisfy                                          *
!   * hy = (2 * pi) / (n * hx)   or    hx = (2 * pi) / (n * hy)       *
!   * h is an input (h = hx if msign < 0 , h = hy if msign > 0)       *
!   * h must be positive                                              *
!   *                                                                 *
!   * ar and ai are two real arrays with dimension n (input and       *
!   * output)                                                         *
!   *                                                                 *
!   *******************************************************************

  implicit none

  double precision, intent(in out) :: ar(*)
  double precision, intent(in out) :: ai(*)
  integer, intent(in)  :: msign
  double precision, intent(in) :: h

  double precision :: a, as, c, s, t, ti, tr, ui, ur, wi, wr
  integer :: i, ip, j, k, l, le, le1, m, n, nm1, nv2
  logical :: debug

1   debug = .false.
1   as = 0.0d0
1   if (msign == 0) then
*>     return
*>   endif

1   if (h <= 0.0d0) then
*>     if (debug) write(*,*) ' *** negative step size in <rcffi>, h = ', h, ' *** '
*>     stop
*>   endif

! *** initialization

1   m = iabs(msign)
1   n = 2**m
1   if (msign > 0) then
1     ar(1) = 0.5d0 * ar(1)
*>   else
*>     as = ar(1) - ai(1)
*>     ar(1) = 0.5d0 * (ar(1) + ai(1))
*>     do i = 2, n
*>       ar(i) = ar(i) + ai(n - i + 2)
*>     enddo
*>     do i = 1, n
*>       ai(i) = 0.0d0
*>     enddo
*>   endif
! *** discrete fast-fourier transform
1   nv2 = n / 2
1   nm1 = n - 1
1   j = 1
1   do i = 1, nm1
4095     if (i < j) then
2016       tr = ar(j)
2016       ar(j) = ar(i)
2016       ar(i) = tr
2016       ti = ai(j)
2016       ai(j) = ai(i)
2016       ai(i) = ti
2016     endif
4095     k = nv2
4095     do while (j > k)
4083       j = j - k
4083       k = k / 2
4083     enddo
4095     j = j + k
4095   enddo
1   do i = 1, n, 2
2048     tr = ar(i + 1)
2048     ar(i + 1) = ar(i) - tr
2048     ar(i) = ar(i) + tr
2048     ti = ai(i + 1)
2048     ai(i + 1) = ai(i) - ti
2048     ai(i) = ai(i) + ti
2048   enddo
1   if (m == 1) return
1   c = 0.0d0
1   s = dble(isign(1, msign))
1   le = 2
1   do l = 2, m
11     wr = c
11     ur = wr
11     wi = s
11     ui = wi
11     c = dsqrt(c * 0.5d0 + 0.5d0)
11     s = wi / (c + c)
11     le1 = le
11     le = le1 + le1
11     do i = 1, n, le
2047       ip = i + le1
2047       tr = ar(ip)
2047       ar(ip) = ar(i) - tr
2047       ar(i)  = ar(i) + tr
2047       ti = ai(ip)
2047       ai(ip) = ai(i) - ti
2047       ai(i)  = ai(i) + ti
2047     enddo
11     do j = 2, le1
4083       do i = j, n, le
20481         ip = i + le1
20481         tr = ar(ip) * ur - ai(ip) * ui
20481         ti = ar(ip) * ui + ai(ip) * ur
20481         ar(ip) = ar(i) - tr
20481         ar(i)  = ar(i) + tr
20481         ai(ip) = ai(i) - ti
20481         ai(i)  = ai(i) + ti
20481       enddo
4083       tr = ur
4083       ur = tr * wr - ui * wi
4083       ui = tr * wi + ui * wr
4083     enddo
11   enddo

! *** correction of the discrete fft, using a trapezoidal approximation
! *** for the variations of the input function

1   c = h
1   if (msign >= 0) then
1     c = c / 3.141592653589793238d0
1     ai(1) = ar(1)
1     do i = 2, n
4095       ai(i) = ar(n - i + 2)
4095     enddo
1   endif
1   ar(1) = c * ar(1)
1   ai(1) = c * ai(1)
1   a = 3.141592653589793238d0 / n
1   wr = dcos(a)
1   wi = dsin(a)
1   t = dsqrt(c)
1   a = a / t
1   c = 0.0d0
1   ur = 1.0d0
1   ui = 0.0d0
1   do i = 2, n
4095     c = c + a
4095     tr = ur
4095     ur = wr * tr - wi * ui
4095     ui = wi * tr + wr * ui
4095     ti = ui / c
4095     tr = ti**2
4095     ar(i) = ar(i) * tr
4095     ai(i) = ai(i) * tr
4095     if (msign <= 0) then
*>       if (as /= 0.0d0) then
*>         ai(i) = ai(i) - as * (t - ur * ti) / (2 * c)
*>       endif
*>     endif
4095   enddo
1   return
end subroutine rcffi

subroutine doboson(t, width, gauss, asym, emin, emax, wmin, wmax, np, p, xout, yout, nout)

! *******************************************************************
! *                                                                 *
! * perform the quantum-mechanical complement to the classical step *
! * of the dielectric theory of eels in specular geometry using a   *
! * suitable thermodynamical average of the quantized surface       *
! * harmonic oscillators                                            *
! *                                                                 *
! * t:      Target temperature                                      *
! * width:  Width of instrumental response (cm**-1)                 *
! * gauss:  Fraction of gaussian for the instrumental response      *
! * asym:   Asymmetry of the instrumental response                  *
! * emin:   Lower loss energy for this computation (cm**-1)         *
! * emax:   Upper loss energy for this computation (cm**-1)         *
! * wmin:   Lower loss energy of the eels computation (cm**-1)      *
! * wmax:   Upper loss energy of the eels computation (cm**-1)      *
! * np:     Number of points of the eels computation                *
! * p:      Intensities of the eels computation                     *
! * xout:   Energies of this computation                            *
! * yout:   Intensities of this computation                         *
! * nout:   Number of points of this computation                    *
! *******************************************************************

  implicit none

  double precision, intent(in) :: t, width, gauss, asym, emin, emax, wmin
  double precision, intent(in out) :: wmax
  integer, intent(in) :: np
  double precision, intent(in) :: p(np)
  double precision, intent(in out) :: xout(nout), yout(nout)
  integer, intent(in out) :: nout

  integer, parameter :: mmax = 14, nmax = 2**mmax
  double precision :: a, a1, a2, alfa, anorm, b, cp2, dwn, fac, fi
  double precision :: fm, fm0, fm1, fp1, fmpic, fp, fp0, fppic, fr, g1, g2
  double precision :: h, pi, sigma, sp2, test, u
  double precision :: wmpic, wppic, wn, x, x1, x2, x3
  double precision :: o1, o2, respon

  external o1, o2

  integer :: i, imax, imin, istep, j, jmin, jmax, m, n
  logical :: picm, picp
  double precision, allocatable :: p1(:), p2(:)
  double precision :: r1(nmax), r2(nmax)

  logical :: debug

  interface
    subroutine rcffi(ar, ai, msign, h)
      double precision, intent(in out) :: ar(*)
      double precision, intent(in out) :: ai(*)
      integer, intent(in)  :: msign
      double precision, intent(in) :: h
    end subroutine rcffi
    subroutine sicot(f, m, h, x0)
      double precision, intent(in out) :: f(*)
      integer, intent(in) :: m
      double precision, intent(in) :: h
      double precision, intent(in) :: x0
    end subroutine sicot
    subroutine sintr(f, msign, h)
      double precision, intent(in out) :: f(*)
      integer, intent(in) :: msign
      double precision, intent(in) :: h
    end subroutine sintr
  end interface  

! remark : the two arrays r1 and r2 are used for fourier
! transforming the user-supplied instrumental response of the
! spectrometer that has to be coded in the external routine
! respon called when the input parameter gauss is < 0 or > 1.
! with 0 <= gauss <= 1, r1 and r2 are not used.

! *** rational approximations for ei(u) * exp(-u) + e1(u) * exp(+u)
! *** in the intervals (0, 1.3) and (1.3, infinity) <accuracy : 4.e-04>
! *** used for fourier transforming half-lorentzian functions


1   debug = .false.
1   data fm1 / 0.0d0 /, fp1 / 0.0d0 /
1   pi = 4 * datan(1.0d0)

1   dwn = (wmax - wmin) / (np - 1)

! *** redefine the frequency interval (0, wmax) to be used for the
! *** fourier transforms

1   if (debug) write(*,*) 'wmin: ', wmin, 'wmax: ', wmax, 'dwn: ', dwn
1   jmin = int(0.5d0 + wmin / dwn)
1   if ((jmin - 0.5d0) * dwn < wmin) then
1     jmin = jmin + 1
1   endif
1   fac = ((jmin - 0.5d0) * dwn - wmin) / dwn
1   jmax = int(0.5d0 + wmax / dwn)
1   test = dmax1(1.5d0 * dabs(emin), 1.5d0 * dabs(emax), 6 * wmax)
1   n = 2
1   do m = 2, mmax
11     n = 2 * n
11     wmax = n * dwn
11     if (wmax >= test) exit
10   enddo
1   if (wmax < test) then
*>     m = mmax
*>     n = nmax
*>     wmax = n * dwn
*>     if (debug) write(*,*) ' +++ n has been fixed at', nmax, ' = 2**', mmax, ' +++'
*>   endif
1   if (debug) then
*>     write(*,*) ' classical spectrum redefined from 0.0 to', wmax
*>     write(*,*) ' step size =', dwn, ', ', n, ' (= 2**', m, ') points'
*>   endif

1   allocate (p1(n))
1   allocate (p2(n))

1   p1 = 0

! *** interpolate pcl on a suitable mesh in (0, wmax)
1   i = 1
1   do j = jmin, min(jmax, n)
325     p1(j) = p(i) + fac * (p(i + 1) - p(i))
325     i = i + 1
325   enddo

1   p2 = p1

! *** characteristic function f(tau)

1   call sicot(p1, m, dwn, 1.39d0 * t)
1   call sintr(p2, m, dwn)
1   h = (pi + pi) / wmax
1   if (gauss < 0.0d0 .or. gauss > 1.0d0) then

! *** broaden the spectrum by convoluting the characteristic function
! *** with a user-supplied response function (arbitrary normalization)

*>     if (debug) write(*,*) '==> switch to a user-supplied instrumental response'
*>     alfa = dmax1(4 * dwn, width)
*>     if (alfa > width) then
*>       if (debug) write(*,*) ' +++ width has been enlarged to', alfa, ' +++'
*>     endif
*>     if (alfa < 10 * dwn) then
*>       if (debug) then
*>         write(*,*) ' ... poor representation of the response function ...'
*>         write(*,*) 'the step size (', dwn, ') should be reduced'
*>       endif
*>     endif
! make a table of the response function
*>     r1(1) = respon(0.0d0, alfa)
*>     r2(1) = r1(1)
*>     do i = 2, n
*>       x = (i - 1) * dwn
*>       r1(i) = respon( x, alfa)
*>       r2(i) = respon(-x, alfa)
*>     enddo
! fourier transform it
*>     call rcffi(r1, r2, -m, dwn)
! normalization of the response function, and convolution
*>     anorm = r1(1)
*>     do i = 1, n
*>       fac = dexp(-p1(i)) / anorm
*>       cp2 = fac * dcos(p2(i))
*>       sp2 = fac * dsin(p2(i))
*>       p1(i) = r1(i) * cp2 + r2(i) * sp2
*>       p2(i) = r2(i) * cp2 - r1(i) * sp2
*>     enddo
*>     alfa = alfa / 2

1   else

! *** broaden the spectrum by convoluting the characteristic function by
! *** a weighted sum of a lorentzian and a gaussian response functions

1     alfa = dmax1(1.5d0 * dwn, width)
1     if (alfa > width) then
*>       if (debug) write(*,*) ' +++ width has been enlarged to', alfa, ' +++'
*>     endif
1     if (alfa > 0.5d0 * wmax / pi) then
*>       stop '*** width is too large, nothing done ***'
*>     endif
1     alfa = 0.5d0 * alfa
1     sigma = alfa / 1.66511d0
1     a1 = (1.0d0 - asym) / 2 * (1.0d0 - gauss)
1     a2 = (1.0d0 + asym) / 2 * (1.0d0 - gauss)
1     if (a1 < 0.0d0 .or. a2 < 0.0d0) then
*>       stop '*** invalid input : asym should be in (-1, +1) ***'
*>     endif
1     g1 = (1.0d0 - asym) * alfa
1     g2 = (1.0d0 + asym) * alfa
1     p1(1) = 1.0d0
1     p2(1) = 0.0d0
1     do i = 2, n
4095       x = (i - 1) * h
4095       fr = 0.0d0
4095       fi = 0.0d0
4095       if (a1 /= 0.0d0) then
4095         x1 = g1 * x
4095         if (x1 <= 100.0d0) then
4095           fr = a1 * dexp(-x1)
4095         endif
4095         if (a1 /= a2) then
4095           if (x1 <= 1.3d0) then
193             fi = a1 * o1(x1)
3902           else
3902             fi = a1 * o2(x1)
3902           endif
4095         endif
4095       endif
4095       if (a2 /= 0.0d0) then
4095         x2 = g2 * x
4095         if (x2 <= 100.0d0) then
4095           fr = fr + a2 * dexp(-x2)
4095         endif
4095         if (a2 /= a1) then
4095           if (x2 <= 1.3d0) then
104             fi = fi - a2 * o1(x2)
3991           else
3991             fi = fi - a2 * o2(x2)
3991           endif
4095         endif
4095       endif
4095       if (gauss /= 0.0d0) then
4095         x3 = sigma * x
4095         if (x3 <= 10.0d0) then
1736           fr = fr + gauss * dexp(-x3**2)
1736         endif
4095       endif
4095       fi = fi / pi
4095       fac = dexp(-p1(i))
4095       cp2 = fac * dcos(p2(i))
4095       sp2 = fac * dsin(p2(i))
4095       p1(i) = fr * cp2 + fi * sp2
4095       p2(i) = fi * cp2 - fr * sp2
4095     enddo
1     if (dabs(fi) > 1.0d-03) then
*>       if (debug) then
*>         write(*,*) ' ... poor representation of the response function ...'
*>         write(*,*) 'the step size (', dwn, ') should be reduced'
*>       endif
*>     endif
1   endif

! *** full eels spectrum

1    call rcffi(p1, p2, m, h)

! *** output

1   istep = max(int(alfa / dwn / 10), 1)
1   nout = 0
1   if (emin <= 0.0d0) then
1     imin = n - int(dabs(emin) / dwn)
1     if ((imin - n) * dwn < emin) then
*>       imin = imin + 1
*>     endif
1     do i = imin, n
251       j = n - i
251       if (mod(j, istep) > 0) cycle
251       x = -j * dwn
251       if (x > emax) exit
251       nout = nout + 1
251       xout(nout)= x
251       yout(nout)= p2(j + 1)
251     enddo
1   endif
1   if (x <= emax) then
1     if (emax >= dwn) then
1       imax = int(emax / dwn) + 1
1       if ((imax - 1) * dwn > emax) then
*>         imax = imax - 1
*>       endif
1       if (imax >= 2) then
1         do i = 2, imax
600           if (mod(i - 1, istep) > 0) cycle
600           x = (i - 1) * dwn
600           if (x < emin) cycle
600           nout = nout + 1
600           xout(nout) = x
600           yout(nout) = p1(i)
600         enddo
1       endif
1     endif
1   endif
1   if (debug) write(*,*) nout, ' values, step size =', istep * dwn

! *** analyze the spectrum

1   if (debug) write(*,*) ' peak location    amplitude'
1   wn = 0.0d0
1   fm = p1(1)
1   fp = p2(1)
1   if (p2(2) < fp .and. p1(2) < fp) then
1     if (debug) write(*, '(f13.2, e13.4)') wn, fp
1   endif
1   fac = 5.0d-06 * fp * dwn
1   imax = 2 + int(dmax1(dabs(emin), dabs(emax)) / dwn)
1   do i = 2, imax
601     fm0 = fm1
601     fp0 = fp1
601     fm1 = fm
601     fp1 = fp
601     fm = p2(i)
601     fp = p1(i)
601     if (i == 2) cycle
600     picm = .false.
600     picp = .false.
600     wn = (i - 1) * dwn
600     if ((fm1 >= fm0) .and. (fm1 >= fm)) then
2       a = (fm1 - fm0) + (fm1 - fm)
2       if (a >= fac) then
2         b = 0.5d0 * ((fm1 - fm0) + 3 * (fm1 - fm))
2         u = b / a
2         wmpic = -wn + u * dwn
2         fmpic = fm + 0.5d0 * b * u
2         picm = .true.
2       endif
2     endif
600     if ((fp1 >= fp0) .and. (fp1 >= fp)) then
2       a = (fp1 - fp0) + (fp1 - fp)
2       if (a >= fac) then
2         b = 0.5d0 * ((fp1 - fp0) + 3 * (fp1 - fp))
2         u = b / a
2         wppic = wn - u * dwn
2         fppic = fp + 0.5d0 * b * u
2         picp = .true.
2         if (picp) then
2           if (picm) then
*>             if (debug) write(*, '(2(f13.2, e13.4, 5x))') wppic, fppic, wmpic, fmpic
2           else
2             if (debug) write(*, '(f13.2, e13.4)') wppic, fppic
2           endif
2         endif
2       endif
2     endif
600     if (picm .and. (.not. picp)) then
2       if (debug) write(*, '(33x, f13.2, e13.4)') wmpic, fmpic
2     endif
600   enddo
1   deallocate (p1)
1   deallocate (p2)
1   return
end subroutine doboson

double precision function o1(u)

  implicit none

  double precision, intent(in) :: u
  double precision :: u2

297   u2 = u**2
297   o1 = -dsinh(u) * dlog(u2) + u * ((0.03114d0 * u2 + 0.41666d0) * u2 + 0.84557d0)

297   return
end function o1

double precision function o2(u)

  implicit none

  double precision, intent(in) :: u
  double precision :: u2

7893   u2 = 1 / u**2
7893   o2 = (((202.91d0 * u2 + 932.21d0) * u2 + 41.740d0) * u2 + 2.0d0) /  &
       (((540.88d0 * u2 + 345.67d0) * u2 + 18.961d0) * u2 + 1.0d0) / u

7893   return
end function o2
