      SUBROUTINE RCFFI(AR,AI,MSIGN,H)                                   
C   ******************************************************************* 
C   *                                                                 * 
C   * USING A RADIX-TWO FAST-FOURIER-TRANSFORM TECHNIQUE, RCFFI       * 
C   * COMPUTES THE FOURIER INTEGRAL TRANSFORM OF A REAL FUNCTION      * 
C   * F(X) OR THE INVERSE FOURIER TRANSFORM OF A COMPLEX FUNCTION     * 
C   * SUCH THAT G(-Y) = CONJ(G(Y)).                                   * 
C   * (ADAPTED FROM CFFT ROUTINE, CERN LIBRARY MEMBER D704)           * 
C   *                                                                 * 
C   * G(Y) = INTEGRAL F(X)*CEXP(-I*X*Y) DX    (MSIGN < 0)             * 
C   *                                                                 * 
C   * F(X) = 1/(2*PI) INTEGRAL G(Y)*CEXP(+I*Y*X) DY    (MSIGN > 0)    * 
C   *                                                                 * 
C   * F(X) IS TABULATED ON THE MESHES OF POINTS DEFINED BY            * 
C   * XJ = J*HX AND -XJ = -J*HX  ,  J = 0, +1, ..., N-2, N-1          * 
C   * AR(J+1) CONTAINS F(+XJ) AND AI(J+1) CONTAINS F(-XJ)             * 
C   * (INPUT WHEN MSIGN < 0 , OUTPUT WHEN MSIGN >0)                   * 
C   * AR(1) MAY BE DIFFERENT FROM AI(1)                               * 
C   *                                                                 * 
C   * G(Y) IS TABULATED ON THE MESH OF POINTS DEFINED BY              * 
C   * YK = K*HY  , K = 0, 1, ..., N-2, N-1                            * 
C   * AR(J+1) CONTAINS REAL(G(YK)) AND AI(J+1) CONTAINS AIMAG(G(YK))  * 
C   * (OUTPUT WHEN MSIGN > 0 , INPUT WHEN MSIGN < 0)                  * 
C   * REMARK :  G(-YK) = CONJ(G(+YK))                                 * 
C   *                                                                 * 
C   * N = 2**IABS(MSIGN)  (MSIGN IS AN INPUT)                         * 
C   *                                                                 * 
C   * THE STEP SIZES SATISFY                                          * 
C   * HY = (2*PI)/(N*HX)   OR    HX = (2*PI)/(N*HY)                   * 
C   * H IS AN INPUT (H = HX IF MSIGN < 0 , H = HY IF MSIGN > 0)       * 
C   * H MUST BE POSITIVE                                              * 
C   *                                                                 * 
C   * AR AND AI ARE TWO REAL ARRAYS WITH DIMENSION N (INPUT AND       * 
C   * OUTPUT)                                                         * 
C   *                                                                 * 
C   ******************************************************************* 
      DIMENSION AR(*),AI(*)                                             
      IF(MSIGN.EQ.0) RETURN                                             
      IF(H.LE.0.0) GOTO 16                                              
C                                                                       
C *** INITIALIZATION                                                    
C                                                                       
      M = IABS(MSIGN)                                                   
      N = 2**M                                                          
      IF(MSIGN.GT.0) GOTO 3                                             
      AS = AR(1)-AI(1)                                                  
      AR(1) = 0.5*(AR(1)+AI(1))                                         
      DO 1 I=2,N                                                        
         AR(I) = AR(I)+AI(N-I+2)                                        
    1 CONTINUE                                                          
      DO 2 I=1,N                                                        
         AI(I) = 0.0                                                    
    2 CONTINUE                                                          
      GOTO 4                                                            
    3 AR(1) = 0.5*AR(1)                                                 
C                                                                       
C *** DISCRETE FAST-FOURIER TRANSFORM                                   
C                                                                       
    4 NV2 = N/2                                                         
      NM1 = N-1                                                         
      J = 1                                                             
      DO 7 I=1,NM1                                                      
         IF(I.LT.J) THEN                                                
            TR = AR(J)                                                  
            AR(J) = AR(I)                                               
            AR(I) = TR                                                  
            TI = AI(J)                                                  
            AI(J) = AI(I)                                               
            AI(I) = TI                                                  
         ENDIF                                                          
         K = NV2                                                        
    5    IF(K.GE.J) GOTO 6                                              
         J = J-K                                                        
         K = K/2                                                        
         GOTO 5                                                         
    6    J = J+K                                                        
    7 CONTINUE                                                          
      DO 8 I=1,N,2                                                      
         TR = AR(I+1)                                                   
         AR(I+1) = AR(I)-TR                                             
         AR(I) = AR(I)+TR                                               
         TI = AI(I+1)                                                   
         AI(I+1) = AI(I)-TI                                             
         AI(I) = AI(I)+TI                                               
    8 CONTINUE                                                          
      IF(M.EQ.1) RETURN                                                 
      C = 0.0                                                           
      S = FLOAT(ISIGN(1,MSIGN))                                         
      LE = 2                                                            
      DO 12 L=2,M                                                       
         WR = C                                                         
         UR = WR                                                        
         WI = S                                                         
         UI = WI                                                        
         C = SQRT(C*0.5+0.5)                                            
         S = WI/(C+C)                                                   
         LE1 = LE                                                       
         LE = LE1+LE1                                                   
         DO 9 I=1,N,LE                                                  
            IP = I+LE1                                                  
            TR = AR(IP)                                                 
            AR(IP) = AR(I)-TR                                           
            AR(I) = AR(I)+TR                                            
            TI = AI(IP)                                                 
            AI(IP) = AI(I)-TI                                           
            AI(I) = AI(I)+TI                                            
    9    CONTINUE                                                       
         DO 11 J=2,LE1                                                  
            DO 10 I=J,N,LE                                              
               IP = I+LE1                                               
               TR = AR(IP)*UR-AI(IP)*UI                                 
               TI = AR(IP)*UI+AI(IP)*UR                                 
               AR(IP) = AR(I)-TR                                        
               AR(I) = AR(I)+TR                                         
               AI(IP) = AI(I)-TI                                        
               AI(I) = AI(I)+TI                                         
   10       CONTINUE                                                    
            TR = UR                                                     
            UR = TR*WR-UI*WI                                            
            UI = TR*WI+UI*WR                                            
   11    CONTINUE                                                       
   12 CONTINUE                                                          
C                                                                       
C *** CORRECTION OF THE DISCRETE FFT, USING A TRAPEZOIDAL APPROXIMATION 
C *** FOR THE VARIATIONS OF THE INPUT FUNCTION                          
C                                                                       
      C = H                                                             
      IF(MSIGN.LT.0) GOTO 14                                            
      C = C/3.1415926                                                   
      AI(1) = AR(1)                                                     
      DO 13 I=2,N                                                       
         AI(I) = AR(N-I+2)                                              
   13 CONTINUE                                                          
   14 AR(1) = C*AR(1)                                                   
      AI(1) = C*AI(1)                                                   
      A = 3.1415926/N                                                   
      WR = COS(A)                                                       
      WI = SIN(A)                                                       
      T = SQRT(C)                                                       
      A = A/T                                                           
      C = 0.0                                                           
      UR = 1.0                                                          
      UI = 0.0                                                          
      DO 15 I=2,N                                                       
         C = C+A                                                        
         TR = UR                                                        
         UR = WR*TR-WI*UI                                               
         UI = WI*TR+WR*UI                                               
         TI = UI/C                                                      
         TR = TI**2                                                     
         AR(I) = AR(I)*TR                                               
         AI(I) = AI(I)*TR                                               
         IF(MSIGN.GT.0) GOTO 15                                         
         IF(AS.NE.0.0) AI(I) = AI(I)-AS*(T-UR*TI)/(C+C)                 
   15 CONTINUE                                                          
      RETURN                                                            
   16 WRITE(6,17) H                                                     
   17 FORMAT(' *** NEGATIVE STEP SIZE IN <RCFFI>, H =',E13.5,' ***')    
      STOP                                                              
      END                                                               
